/*
 *
 *    ******************************************************************************
 *    *
 *    * This program and the accompanying materials are made available under the
 *    * terms of the Apache License, Version 2.0 which is available at
 *    * https://www.apache.org/licenses/LICENSE-2.0.
 *    *
 *    *  See the NOTICE file distributed with this work for additional
 *    *  information regarding copyright ownership.
 *    * Unless required by applicable law or agreed to in writing, software
 *    * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 *    * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 *    * License for the specific language governing permissions and limitations
 *    * under the License.
 *    *
 *    * SPDX-License-Identifier: Apache-2.0
 *    *****************************************************************************
 *
 */

package net.brutex.ai.nd4j.tests;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import lombok.extern.slf4j.Slf4j;
import org.deeplearning4j.datasets.iterator.INDArrayDataSetIterator;
import org.deeplearning4j.datasets.iterator.impl.IrisDataSetIterator;
import org.deeplearning4j.datasets.iterator.impl.MnistDataSetIterator;
import org.deeplearning4j.nn.conf.NeuralNetConfiguration;
import org.deeplearning4j.nn.conf.layers.DenseLayer;
import org.deeplearning4j.nn.conf.layers.OutputLayer;
import org.deeplearning4j.nn.multilayer.MultiLayerNetwork;
import org.junit.jupiter.api.Test;
import org.nd4j.common.primitives.Pair;
import org.nd4j.evaluation.classification.Evaluation;
import org.nd4j.linalg.activations.Activation;
import org.nd4j.linalg.api.buffer.DataType;
import org.nd4j.linalg.api.ndarray.INDArray;
import org.nd4j.linalg.factory.Nd4j;
import org.nd4j.linalg.lossfunctions.LossFunctions;

@Slf4j
public class ExploreParamsTest {

  @Test
  public void testParam() {
    NeuralNetConfiguration conf =
        NeuralNetConfiguration.builder()
            .seed(12345)
                .dataType(DataType.DOUBLE)
            .layer(
                DenseLayer.builder().nIn(4).nOut(30).name("1. Dense").activation(Activation.TANH))
           .layer(DenseLayer.builder().nIn(30).nOut(10).name("2. Dense"))
            //  .layer(FrozenLayer.builder(DenseLayer.builder().nOut(6).build()).build())

            .layer(
                OutputLayer.builder()
                    .nOut(3)
                    .lossFunction(LossFunctions.LossFunction.MSE)
                    .activation(Activation.SOFTMAX))
            .build();
    MultiLayerNetwork nn = new MultiLayerNetwork(conf);
    nn.init();
    log.info(nn.summary());
    // INDArray input = Nd4j.rand(10,4);
    INDArray labels = Nd4j.zeros(9, 3);

    INDArray input =
        Nd4j.create(
            new double[][] {
              {5.15, 3.5, 1.4, 0.21},     // setosa
              {4.9, 3.2, 1.4, 0.2},       // setosa
              {4.7, 3.2, 1.23, 0.2},      // setosa
              {7, 3.25, 4.7, 1.41},       // versicolor
              {6.4, 3.2, 4.54, 1.5},      // versicolor
              {6.9, 3.1, 4.92, 1.5},      // versicolor
              {7.7, 3, 6.1, 2.3},         // virginica
              {6.3, 3.4, 5.6, 2.45},      // virginica
              {6.4, 3.12, 5.5, 1.8}       // virginica
            });

    labels.putScalar(0, 1);
    labels.putScalar(3, 1);
    labels.putScalar(6, 1);
    labels.putScalar(10, 1);
    labels.putScalar(13, 1);
    labels.putScalar(16, 1);
    labels.putScalar(20, 1);
    labels.putScalar(23, 1);
    labels.putScalar(26, 1);

    IrisDataSetIterator iter = new IrisDataSetIterator();
    //Iterable<Pair<INDArray, INDArray>> it = List.of(new Pair<INDArray, INDArray>(input, labels));
    List l = new ArrayList<>();
    for (int i=0; i< input.rows(); i++) {
      l.add(new Pair(input.getRow(i), labels.getRow(i)));
    }
    Iterable<Pair<INDArray, INDArray>> it = l;
    INDArrayDataSetIterator diter = new INDArrayDataSetIterator(it, 1);

    for (int i = 0; i < 100; i++) {
      // nn.fit(input, labels);
      // nn.fit( input, labels);
      nn.fit(diter);
      // nn.feedForward(input);
      if(i%20==0) log.info("Score: {}", nn.getScore());
    }

    Evaluation eval = nn.evaluate(iter, List.of("setosa", "vericolor", "virginica"));

    log.info("\n{}", eval.stats());
  }

  @Test
  public void testParam2() throws IOException {
    NeuralNetConfiguration conf =
            NeuralNetConfiguration.builder()
                    .seed(12345)
                    .layer(
                            DenseLayer.builder().nIn(784).nOut(20).name("1. Dense"))
                    .layer(DenseLayer.builder().nIn(20).nOut(10).name("2. Dense"))
                    .layer(
                            OutputLayer.builder()
                                    .nOut(10)
                                    .lossFunction(LossFunctions.LossFunction.MSE)
                                    .activation(Activation.SOFTMAX))
                    .build();
    MultiLayerNetwork nn = new MultiLayerNetwork(conf);
    nn.init();
    log.info(nn.summary());

    NeuralNetConfiguration conf2 =
            NeuralNetConfiguration.builder()
                    .seed(12345)
                    .layer(
                            DenseLayer.builder().nIn(784).nOut(20).name("1. Dense").dropOut(0.7))
                    .layer(DenseLayer.builder().nIn(20).nOut(10).name("2. Dense"))
                    .layer(
                            OutputLayer.builder()
                                    .nOut(10)
                                    .lossFunction(LossFunctions.LossFunction.MSE)
                                    .activation(Activation.SOFTMAX))
                    .build();
    MultiLayerNetwork nn2 = new MultiLayerNetwork(conf2);
    nn2.init();
    log.info(nn2.summary());



    MnistDataSetIterator iter = new MnistDataSetIterator(10, 500);
    MnistDataSetIterator iter2 = new MnistDataSetIterator(10, 50);


    for (int i = 0; i < 200; i++) {
      nn.fit(iter);
      nn2.fit(iter);
      if(i%20==0) log.info("Score: {} vs. {}", nn.getScore(), nn2.getScore());
    }

    Evaluation eval = nn.evaluate(iter2);
    Evaluation eval2 = nn2.evaluate(iter2);

    log.info("\n{} \n{}", eval.stats(), eval2.stats());
  }
}
