/*
 *  ******************************************************************************
 *  *
 *  *
 *  * This program and the accompanying materials are made available under the
 *  * terms of the Apache License, Version 2.0 which is available at
 *  * https://www.apache.org/licenses/LICENSE-2.0.
 *  *
 *  *  See the NOTICE file distributed with this work for additional
 *  *  information regarding copyright ownership.
 *  * Unless required by applicable law or agreed to in writing, software
 *  * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 *  * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 *  * License for the specific language governing permissions and limitations
 *  * under the License.
 *  *
 *  * SPDX-License-Identifier: Apache-2.0
 *  *****************************************************************************
 */

package org.deeplearning4j.nn.conf.layers;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import lombok.*;
import lombok.experimental.SuperBuilder;
import org.deeplearning4j.nn.params.PretrainParamInitializer;
import org.nd4j.linalg.lossfunctions.LossFunctions;

@ToString(callSuper = true)
@EqualsAndHashCode(callSuper = true)
@JsonIgnoreProperties("pretrain")
@SuperBuilder
public abstract class BasePretrainNetwork extends FeedForwardLayer {
  @Builder.Default @Getter
  protected LossFunctions.LossFunction lossFunction =
      LossFunctions.LossFunction.RECONSTRUCTION_CROSSENTROPY;

  @Builder.Default @Getter protected double visibleBiasInit = 0.0;

  @Override
  public boolean isPretrainParam(String paramName) {
    return PretrainParamInitializer.VISIBLE_BIAS_KEY.equals(paramName);
  }
}
