/*
 *
 *    ******************************************************************************
 *    *
 *    * This program and the accompanying materials are made available under the
 *    * terms of the Apache License, Version 2.0 which is available at
 *    * https://www.apache.org/licenses/LICENSE-2.0.
 *    *
 *    *  See the NOTICE file distributed with this work for additional
 *    *  information regarding copyright ownership.
 *    * Unless required by applicable law or agreed to in writing, software
 *    * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 *    * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 *    * License for the specific language governing permissions and limitations
 *    * under the License.
 *    *
 *    * SPDX-License-Identifier: Apache-2.0
 *    *****************************************************************************
 *
 */

package net.brutex.ai.dnn.serde;

import static org.junit.jupiter.api.Assertions.*;

import lombok.extern.slf4j.Slf4j;
import org.deeplearning4j.nn.conf.NeuralNetConfiguration;
import org.deeplearning4j.nn.conf.distribution.NormalDistribution;
import org.deeplearning4j.nn.conf.layers.DenseLayer;
import org.deeplearning4j.nn.conf.layers.OutputLayer;
import org.deeplearning4j.nn.weights.WeightInit;
import org.junit.jupiter.api.Test;
import org.nd4j.linalg.lossfunctions.LossFunctions;
@Slf4j
class NeuralNetConfigurationSerdeTest {

    @Test
    void toYaml() {
      }

      @Test void toJson0() {
          final var conf = NeuralNetConfiguration.builder()
                  .weightInit(new NormalDistribution(3, 2))
                  .layer(DenseLayer.builder().nIn(100).nOut(30).build())
                  .layer(OutputLayer.builder().lossFunction(LossFunctions.LossFunction.SQUARED_LOSS).build())
                  .build();
          log.info(conf.toJson());
      }
    @Test
    void toJson() {
        final var conf = NeuralNetConfiguration.builder()
                .weightInit(new NormalDistribution(3, 2))
                .layer(DenseLayer.builder().nIn(100).nOut(30).build())
                .layer(OutputLayer.builder().lossFunction(LossFunctions.LossFunction.SQUARED_LOSS).build())
                .build();
        assertEquals(conf.toJson(), NeuralNetConfiguration.fromJson(conf.toJson()).toJson());
      }

    @Test
    void toJson2() {
        final var conf = NeuralNetConfiguration.builder()
                .weightInit(WeightInit.IDENTITY)
                .layer(DenseLayer.builder().nIn(100).nOut(30).build())
                .layer(OutputLayer.builder().lossFunction(LossFunctions.LossFunction.SQUARED_LOSS).build())
                .build();
        assertEquals(conf.toJson(), NeuralNetConfiguration.fromJson(conf.toJson()).toJson());
    }

    @Test
    void fromJson() {
    final String json =
        "{\n"
            + "  \"activation\" : null,\n"
            + "  \"allParamConstraints\" : [ ],\n"
            + "  \"backpropType\" : \"Standard\",\n"
            + "  \"biasConstraints\" : [ ],\n"
            + "  \"biasInit\" : 0.0,\n"
            + "  \"biasUpdater\" : null,\n"
            + "  \"cacheMode\" : \"NONE\",\n"
            + "  \"constrainWeights\" : [ ],\n"
            + "  \"convolutionMode\" : \"Truncate\",\n"
            + "  \"cudnnAlgoMode\" : \"PREFER_FASTEST\",\n"
            + "  \"dampingFactor\" : 100.0,\n"
            + "  \"dataType\" : \"FLOAT\",\n"
            + "  \"epochCount\" : 0,\n"
            + "  \"gainInit\" : 1.0,\n"
            + "  \"gradientNormalization\" : \"None\",\n"
            + "  \"gradientNormalizationThreshold\" : 0.0,\n"
            + "  \"idropOut\" : null,\n"
            + "  \"inferenceWorkspaceMode\" : \"ENABLED\",\n"
            + "  \"initCalled\" : false,\n"
            + "  \"innerConfigurations\" : [ {\n"
            + "    \"org.deeplearning4j.nn.conf.layers.DenseLayer\" : {\n"
            + "      \"activation\" : [ \"org.nd4j.linalg.activations.Activation\", \"IDENTITY\" ],\n"
            + "      \"biasInit\" : 0.0,\n"
            + "      \"constrainWeights\" : [ ],\n"
            + "      \"gainInit\" : 0.0,\n"
            + "      \"gradientNormalization\" : \"None\",\n"
            + "      \"gradientNormalizationThreshold\" : 1.0,\n"
            + "      \"hasBias\" : true,\n"
            + "      \"hasLayerNorm\" : false,\n"
            + "      \"nIn\" : 100,\n"
            + "      \"nOut\" : 30,\n"
            + "      \"regularization\" : [ ],\n"
            + "      \"regularizationBias\" : [ ],\n"
            + "      \"type\" : \"UNKNOWN\",\n"
            + "      \"variables\" : [ ]\n"
            + "    }\n"
            + "  }, {\n"
            + "    \"org.deeplearning4j.nn.conf.layers.OutputLayer\" : {\n"
            + "      \"activation\" : [ \"org.nd4j.linalg.activations.Activation\", \"IDENTITY\" ],\n"
            + "      \"biasInit\" : 0.0,\n"
            + "      \"constrainWeights\" : [ ],\n"
            + "      \"gainInit\" : 0.0,\n"
            + "      \"gradientNormalization\" : \"None\",\n"
            + "      \"gradientNormalizationThreshold\" : 1.0,\n"
            + "      \"hasBias\" : true,\n"
            + "      \"lossFunction\" : {\n"
            + "        \"@class\" : \"org.nd4j.linalg.lossfunctions.impl.LossMSE\"\n"
            + "      },\n"
            + "      \"nIn\" : 0,\n"
            + "      \"nOut\" : 0,\n"
            + "      \"regularization\" : [ ],\n"
            + "      \"regularizationBias\" : [ ],\n"
            + "      \"type\" : \"UNKNOWN\",\n"
            + "      \"variables\" : [ ]\n"
            + "    }\n"
            + "  } ],\n"
            + "  \"inputPreProcessors\" : { },\n"
            + "  \"inputType\" : null,\n"
            + "  \"iterationCount\" : 0,\n"
            + "  \"maxNumLineSearchIterations\" : 5,\n"
            + "  \"miniBatch\" : false,\n"
            + "  \"minimize\" : true,\n"
            + "  \"name\" : \"Anonymous INeuralNetworkConfiguration\",\n"
            + "  \"netWideVariables\" : [ ],\n"
            + "  \"optimizationAlgo\" : \"STOCHASTIC_GRADIENT_DESCENT\",\n"
            + "  \"overrideNinUponBuild\" : true,\n"
            + "  \"regularization\" : [ ],\n"
            + "  \"regularizationBias\" : [ ],\n"
            + "  \"seed\" : 12345,\n"
            + "  \"stepFunction\" : null,\n"
            + "  \"tbpttBackLength\" : 20,\n"
            + "  \"tbpttFwdLength\" : 20,\n"
            + "  \"trainingWorkspaceMode\" : \"ENABLED\",\n"
            + "  \"updater\" : {\n"
            + "    \"@class\" : \"org.nd4j.linalg.learning.config.Sgd\",\n"
            + "    \"learningRate\" : 0.001\n"
            + "  },\n"
            + "  \"validateOutputLayerConfig\" : true,\n"
            + "  \"validateTbpttConfig\" : true,\n"
            + "  \"weightInit\" : {\n"
            + "    \"@class\" : \"org.deeplearning4j.nn.weights.WeightInitIdentity\",\n"
            + "    \"scale\" : null\n"
            + "  },\n"
            + "  \"weightNoise\" : null\n"
            + "}";
        final var conf = NeuralNetConfiguration.builder()
                .weightInit(WeightInit.IDENTITY)
                .layer(DenseLayer.builder().nIn(100).nOut(30).build())
                .layer(OutputLayer.builder().lossFunction(LossFunctions.LossFunction.SQUARED_LOSS).build())
                .seed(12345)
                .build();
        NeuralNetConfiguration conf2 = NeuralNetConfiguration.fromJson(json);
        assertEquals(conf.toJson(),conf2.toJson());
        assertEquals(conf, conf2);
    }

}